/**
   This program demonstrates several ways to rotate a cube.
*/
import java.awt.Color;

public class RotatingCube
{
   public static void main(String[] args)
   {
      // Here are the vertices of the standard cube.
      Vertex[] v = { new Vertex(0, 0, 0), // count from 0 to to 7 in binary
                     new Vertex(0, 0, 1),
                     new Vertex(0, 1, 0),
                     new Vertex(0, 1, 1),
                     new Vertex(1, 0, 0),
                     new Vertex(1, 0, 1),
                     new Vertex(1, 1, 0),
                     new Vertex(1, 1, 1) };

      // Make 12 line segments.
      LineSegment[] ls = { new LineSegment(v[0], v[1]),  // bottom edges
                           new LineSegment(v[1], v[5]),
                           new LineSegment(v[5], v[4]),
                           new LineSegment(v[4], v[0]),

                           new LineSegment(v[2], v[3]),  // top edges
                           new LineSegment(v[3], v[7]),
                           new LineSegment(v[7], v[6]),
                           new LineSegment(v[6], v[2]),

                           new LineSegment(v[0], v[2]),  // vertical edges
                           new LineSegment(v[1], v[3]),
                           new LineSegment(v[5], v[7]),
                           new LineSegment(v[4], v[6]) };

      // Create two empty Model objects.
      Model m1 = new Model();
      Model m2 = new Model();

      // Put the line segments in the Models
      // (and give each line segment a random color).
      for (int i = 0; i < ls.length; i++)
      {
         ls[i].vertexList.get(0).setColorRandom();
         ls[i].vertexList.get(1).setColor( ls[i].vertexList.get(0).getColor() );
         m1.primitiveList.add( ls[i] );
         m2.primitiveList.add( ls[i] );
      }

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add( m1 );
      scene.modelList.add( m2 );

      // Create a Camera object.
      Camera camera = new Camera();

      // Give the Camera a view matrix.
      camera.view2Identity();

      // Set up the camera's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();


      // Demostrate three ways to make a cube rotate.

      // (1)
//      m1.model2Identity();
//      m1.modelTranslate( 2, 0, 0);

      for (int i = 0; i <= 72; i++)
      {
         // still (1)
//         m1.modelRotate(5, 0, 0, 1);  // an accumulating rotation

         // (2)
         m1.model2Identity();
         m1.modelTranslate( 2, 0, 0);
         m1.modelRotate(i*5, 0, 0, 1);

         // (3)
         m2.model2Identity();
         m2.modelRotate(i*5, 0, 0, 1);
         m2.modelTranslate( 2, 0, 0);

         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.lightGray);
         // Render our scene into the frame buffer.
         renderer.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File(String.format("PPM_RotatingCube_Frame%03d.ppm", i));
      }
   }
}//RotatingCube