/*

*/
import java.awt.Color;

public class PrismClient0
{
   public static void main(String[] args)
   {
      double sqrt3 = Math.sqrt(3.0);

      // Create a Prism object.
      Model m0 = new PrismModel(2, Math.PI/4.0);
      m0.doLighting = false;
      // Give each triangle a random color.
      for (Triangle t : m0.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }

      // Create four new Prism objects from m0.
      // right
      Model m1 = new Model(m0);  // clone model m0
      for (Triangle t : m0.triangleList)
         m1.triangleList.add(t);
      m1.modelTranslate(2+0.5/sqrt3, 0, 0);
      // left
      Model m2 = new Model(m0);  // clone model m0
      for (Triangle t : m0.triangleList)
         m2.triangleList.add(t);
      m2.modelRotate(180, 0,0,1);
      m2.modelTranslate(2+0.5/sqrt3, 0, 0);
      // top
      Model m3 = new Model(m0);  // clone model m0
      for (Triangle t : m0.triangleList)
         m3.triangleList.add(t);
      m3.modelRotate(90, 0,0,1);
      m3.modelTranslate(2+0.5/sqrt3, 0, 0);
      // bottom
      Model m4 = new Model(m0);  // clone model m0
      for (Triangle t : m0.triangleList)
         m4.triangleList.add(t);
      m4.modelRotate(-90, 0,0,1);
      m4.modelTranslate(2+0.5/sqrt3, 0, 0);

      Model floor = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      floor.doLighting = false;
      floor.doBackFaceCulling = false;
      floor.modelTranslate(0, -4, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(floor);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);


      // Create a rendering pipeline,
      SceneRender renderer = new SceneRender(scene, fb);  // default pipeline
      // Check the order of the pipeline stages            .
      System.err.println( renderer );

      // Render our scene into the frame buffer.
      renderer.renderScene();
      // Save the resulting image in a file.
      String basename = "PPM_PrismClient_0_Frame_";
      fb.dumpFB2File( String.format(basename + "%02d.ppm", 0) );

      // Spin the model 360 degrees arond two axes.
      for (int i = 1; i <= 72; i++)
      {
         // Rotate the Prisms by 5 degrees.
         m1.modelRotate( 5, 1,0,0);  // right side
         m2.modelRotate( 5, 1,0,0);  // left side
         m3.modelRotate( 5, 1,0,0);  // top
         m4.modelRotate( 5, 1,0,0);  // bottom

         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         renderer.renderScene();
         fb.dumpFB2File(String.format(basename + "%02d.ppm", i));
      }
   }
}