/*
   This example program is important because it gives us a simple example with which to analyze memory allocation in the rendering algorithms (see the example MainTriangle.java in the Clients folder).

   In this version of the renderer, we have "higher order" geometric primitives like LineStrip and LineLoop. We can solve our memory allocation problem for a triangle by using one of these more sophisticated primitives.



*/
import java.awt.Color;

public class TriangleLineLoop
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Add the geometry to the Model.
      Vertex[] v = new Vertex[3];
      v[0] = new Vertex(3.0, 0.0, 0.0);
      v[1] = new Vertex(0.0, 3.0, 0.0);
      v[2] = new Vertex(0.0, 0.0, 3.0);

      // Give each vertex an individual color.
      v[0].setColor(new Color(255, 0, 0));
      v[1].setColor(new Color(0, 255, 0));
      v[2].setColor(new Color(0, 0, 255));

      LineLoop lineLoop = new LineLoop(v);
      model.addPrimitive(lineLoop);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Create a Camera object.
      Camera camera = new Camera();
/*
      // Set up the camera's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);
*/
      // Set up the camera's view frustum.
      double left   = -1.0;
      double right  =  1.0;
      double top    =  1.0;
      double bottom = -1.0;
      double near   =  2.0;
      double far    =  20.0;
      camera.projFrustum(left, right, bottom, top, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=6;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);
      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      System.err.println( renderer.toString() );

      System.err.println( scene.toString() );

      // Render our scene into the frame buffer.
      renderer.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_TriangleLineLoop_Frame%02d.ppm", 0) );

      // Spin the model 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the model by 10 degrees.
         scene.camera.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         renderer.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_TriangleLineLoop_Frame%02d.ppm", i));
      }
   }
}