/**
   A LineSegment object has two Vertex objects that represent the endpoints of the line segment.
*/
import java.util.LinkedList;
import java.util.List;

public class LineSegment extends Primitive
{
   //public Vertex[] v = new Vertex[2]; // the vertices of this line segment

   public LineSegment()
   {
      super();
   }


   /**
      Construct a LineSegment from an array of vertices.
   */
   public LineSegment(Vertex[] vArray)
   {
      super(vArray);
   }


   /**
      Construct a LineSegment from a List of vertices.
   */
   public LineSegment(List<Vertex> vList)
   {
      super(vList);
   }


   /**
      Create a LineSegment object whose vertices are copies of the two given Vertex objects.
   */
   public LineSegment(Vertex v0, Vertex v1)
   {
      super();

      this.vertexList.add( new Vertex(v0) );  // make copies of each Vertex!
      this.vertexList.add( new Vertex(v1) );
   }


   /**
      Create a LineSegment object whose vertices are copies of the two vertices from the given LineSegment object.
   */
   public LineSegment(LineSegment ls) // a "copy constructor"
   {
      super();

      this.vertexList.add( new Vertex(ls.vertexList.get(0)) );  // make copies of each Vertex!
      this.vertexList.add( new Vertex(ls.vertexList.get(1)) );
   }


   /**
      Make a deep copy of this primitive.
   */
   public Primitive makeCopy()
   {
      return new LineSegment(this);
   }


   /**
      Implement the assemble() method from the abstract base class.

      A LineSegment should return a List<LineSegment>
      object containing just this one LineSgement object.
   */
   public List<LineSegment> assemble()
   {
      List<LineSegment> result = new LinkedList<LineSegment>();
      result.add( this );
      return result;
   }


   /**
      For debugging.
   */
   public String toString()
   {
      String result = "Line Segment:\n";
      result += this.vertexList.get(0).toString();
      result += this.vertexList.get(1).toString();
      return result;
   }

   /**
      For debugging. What this method prints out
      depends on the value of the parameter vertexFields.

      The value of vertexFields is treated as a bit field.
      The bits have the following meanings.
        1 => print vertex coordinates
        2 => print r g b values
   */
   public String toString(int vertexFields)
   {
      String result = "Line Segment:\n";
      result += this.vertexList.get(0).toString(vertexFields);
      result += this.vertexList.get(1).toString(vertexFields);
      return result;
   }
}