/**
   http://docs.oracle.com/javase/6/docs/api/java/util/List.html

   The List interface provides four methods for positional (indexed) access to list elements. Lists (like Java arrays) are zero based. Note that these operations may execute in time proportional to the index value for some implementations (the LinkedList class, for example). Thus, iterating over the elements in a list is typically preferable to indexing through it if the caller does not know the implementation.

   The List interface provides a special iterator, called a ListIterator, that allows element insertion and replacement, and bidirectional access in addition to the normal operations that the Iterator interface provides.
*/
import java.util.ListIterator;

public class P1_Model2World_Examples extends PipelineStage
{
   public P1_Model2World_Examples(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each geometric primitive in the model, transform each vertex
      in the primitive from model coordinates to world coordinates.
   */
   public Model processModel(Model model)
   {
      Matrix modelMatrix = model.modelMatrix;

      // Transform each geometric primitive in the model's primitive list.
      for (Primitive p : model.primitiveList)
      {
         // Transform each vertex in the geometric primitive.

         // Example 1
         ListIterator<Vertex> itr = p.vertexList.listIterator();
         while (itr.hasNext())
         {
            Vertex v = itr.next();
            itr.set( modelMatrix.times(v) );
         }

         // Example 2
         for (int i = 0; i < p.vertexList.size(); i++)
         {
            Vertex v = p.vertexList.get(i);            // May be very inefficient!
            p.vertexList.set(i, modelMatrix.times(v)); // May be very inefficient!
         }

         // Example 3
         for (Vertex v : p.vertexList)
         {
            v = modelMatrix.times(v);   // WRONG!
         }
      }
      return model;
   }
}