/**
   Convert all the model's primitives into LineSegment objects.

   For each Model in a Scene, walk the Model's list of Primitive objects.
   For each Primitve, call its assemble() method and get its list of
   LineSegment objects. Gather all the LineSegment objects into a list,
   and then make that list into the model's list of primitives (so the
   model's final list of primitives contains only LineSegment objects,
   which is exactly what the next stage, the clipper, knows how to clip).
*/
import java.util.List;
import java.util.LinkedList;

public class P5_PrimitiveAssembly extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P5_PrimitiveAssembly(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each primitive in the model, convert the primitive
      into an appropriate list of LineSegment objects.
   */
   public Model processModel(Model model)
   {
      List<Primitive> lineSegments = new LinkedList<Primitive>();

      for (Primitive p : model.primitiveList)
      {
         lineSegments.addAll( p.assemble() );
      }

      model.primitiveList = lineSegments;

      return model;
   }
}