/**
   This can read a simple OBJ file.
*/
import java.util.regex.*;
import java.util.ArrayList;
import java.util.Scanner;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

public class ObjSimpleModel extends Model
{
   public ObjSimpleModel(File modelFile)
   {
      super();

      // Open the OBJ file.
      String objName = null;
      FileInputStream fis = null;
      try
      {
         objName = modelFile.getCanonicalPath();
         fis = new FileInputStream( modelFile );
      }
      catch (FileNotFoundException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }

      // Get the geometry from the OBJ file.
      ArrayList<Vertex> verts = new ArrayList<Vertex>();
      try
      {
         // Pattern for parsing lines that start with "f"
         Pattern p = Pattern.compile("^(\\d*)[/]?(\\d*)[/]?(\\d*)");

         Scanner scanner = new Scanner(fis);
         while ( scanner.hasNext() )
         {
            String token = scanner.next();
            if ( token.startsWith("#") )
            {
               scanner.nextLine(); // skip over the comment
            }
            else if ( token.startsWith("v") )
            {
               double x = scanner.nextDouble();
               double y = scanner.nextDouble();
               double z = scanner.nextDouble();
               Vertex v = new Vertex(x, y, z);
               verts.add( v );
            }// parse vertex
            else if ( token.startsWith("f") )
            {
               Primitive lineLoop = new LineLoop();
               // tokenize the rest of the line
               String restOfLine = scanner.nextLine();
               Scanner scanner2 = new Scanner( restOfLine );
               // parse a vertex (if there is one) and add it to the LineLoop
               while (scanner2.hasNext())
               {
                  // parse a "v/vt/vn" group
                  String faceGroup = scanner2.next();
                  Matcher m = p.matcher( faceGroup );
                  if ( m.find() )
                  {
                     int vertexIndex = Integer.parseInt( m.group(1) );
                     String vt = m.group(2);  // don't need
                     String vn = m.group(3);  // don't need
                     Vertex v = verts.get(vertexIndex - 1);
                     lineLoop.addVertex( v );
                  }
                  else
                     System.err.println("Error: bad face: " + faceGroup);
               }
               // add the LineLoop around this face to the model
               this.addPrimitive( lineLoop );
            }// parse face
         }// parse one line
         fis.close();
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not read OBJ file: %s\n", objName);
         System.exit(-1);
      }
   }
}//ObjSimpleModel